require 'dbc'

DBC.specification "Stack" do
  contract :empty? do
    query
  end

  invariant { empty? == (depth == 0) }

  contract :push do
    remember :depth
    post("Stack size increases") { depth == dbc_old(:depth) + 1 }
    post("Top is pushed item")   { |result, item| top == item }
  end

  contract :pop do
    remember :depth
    pre("Stack is not empty") { ! empty? }
    post("Stack size is decreased") { depth == dbc_old(:depth) -1 }
  end

  example "Empty Stack" do |stack|
  end

  example "Pushing Items" do |stack|
    stack.push(1)
  end

  example "Popping items from a stack" do |stack|
    stack.push(1)
    stack.pop
  end
end

class MyStack
  def initialize
    @items = []
  end

  def depth
    @items.size
  end

  def empty?
    depth == 0
  end

  def top
    @items.last
  end

  def pop
    @items.pop
  end

  def push(item)
    @items << item
  end
end

MyStack.obeys_contracts_for "Stack"
